@@ -1,52 +1,56 @@
-/**
- @file bzrtp.h
-
- @brief Public entry points to the ZRTP implementation
-
- @author Johan Pascal
-
- @copyright Copyright (C) 2014 Belledonne Communications, Grenoble, France
- 
- This program is free software; you can redistribute it and/or
- modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 2
- of the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/*
+ * Copyright (c) 2014-2019 Belledonne Communications SARL.
+ *
+ * This file is part of bzrtp.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef BZRTP_H
 #define BZRTP_H
 
 #include <stdint.h>
 #include "bctoolbox/crypto.h"
+#include "bctoolbox/port.h"
 
 #ifdef _MSC_VER
 	#ifdef BZRTP_STATIC
 		#define BZRTP_EXPORT
-	#else
+	#else /* BZRTP_STATIC */
 		#ifdef BZRTP_EXPORTS
 			#define BZRTP_EXPORT __declspec(dllexport)
-		#else
+		#else /* BZRTP_EXPORTS */
 			#define BZRTP_EXPORT __declspec(dllimport)
-		#endif
-	#endif
-#else
+		#endif /* BZRTP_EXPORTS */
+	#endif /* BZRTP_STATIC */
+
+	#ifndef BZRTP_DEPRECATED
+		#define BZRTP_DEPRECATED __declspec(deprecated)
+	#endif /* BZRTP_DEPRECATED */
+#else /* _MSC_VER*/
 	#define BZRTP_EXPORT __attribute__ ((visibility ("default")))
-#endif
+
+	#ifndef BZRTP_DEPRECATED
+		#define BZRTP_DEPRECATED __attribute__ ((deprecated))
+	#endif /* BZRTP_DEPRECATED */
+#endif /* _MSC_VER*/
 
 /**
  * Define different types of crypto functions */
 #define ZRTP_HASH_TYPE			0x01
-#define ZRTP_CIPHERBLOCK_TYPE 	0x02
+#define ZRTP_CIPHERBLOCK_TYPE 		0x02
 #define ZRTP_AUTHTAG_TYPE		0x04
-#define ZRTP_KEYAGREEMENT_TYPE	0x08
+#define ZRTP_KEYAGREEMENT_TYPE		0x08
 #define ZRTP_SAS_TYPE			0x10
 
 /**
@@ -76,13 +80,15 @@
  * as it is used to easily sort them from faster(DH2k) to slower(EC52)
  */
 #define ZRTP_KEYAGREEMENT_DH2k	0x41
-#define ZRTP_KEYAGREEMENT_EC25	0x42
-#define ZRTP_KEYAGREEMENT_DH3k	0x43
-#define ZRTP_KEYAGREEMENT_EC38	0x44
-#define ZRTP_KEYAGREEMENT_EC52	0x45
+#define ZRTP_KEYAGREEMENT_X255	0x42
+#define ZRTP_KEYAGREEMENT_EC25	0x43
+#define ZRTP_KEYAGREEMENT_X448	0x44
+#define ZRTP_KEYAGREEMENT_DH3k	0x45
+#define ZRTP_KEYAGREEMENT_EC38	0x46
+#define ZRTP_KEYAGREEMENT_EC52	0x47
 
-#define ZRTP_KEYAGREEMENT_Prsh	0x46
-#define ZRTP_KEYAGREEMENT_Mult	0x47
+#define ZRTP_KEYAGREEMENT_Prsh	0x4e
+#define ZRTP_KEYAGREEMENT_Mult	0x4f
 
 #define ZRTP_SAS_B32			0x51
 #define ZRTP_SAS_B256			0x52
@@ -115,8 +121,13 @@ typedef struct bzrtpSrtpSecrets_struct  {
 	uint8_t keyAgreementAlgo; /**< The key agreement algo selected during ZRTP negotiation */
 	uint8_t sasAlgo; /**< The SAS rendering algo selected during ZRTP negotiation */
 	uint8_t cacheMismatch; /**< Flag set to 1 in case of ZRTP cache mismatch, may occurs only on first channel(the one computing SAS) */
+	uint8_t auxSecretMismatch; /**< Flag set to BZRTP_AUXSECRET_MATCH, BZRTP_AUXSECRET_MISMATCH or BZRTP_AUXSECRET_UNSET, may occurs only on first channel(the one computing SAS), in case of mismatch it may be ignored and we can still validate the SAS */
 } bzrtpSrtpSecrets_t;
 
+/* define auxSecretMismatch flag codes */
+#define BZRTP_AUXSECRET_MATCH		0x00
+#define BZRTP_AUXSECRET_MISMATCH	0x01
+#define BZRTP_AUXSECRET_UNSET		0x02
 
 /* define message levels */
 #define BZRTP_MESSAGE_ERROR	0x00
@@ -167,6 +178,7 @@ typedef struct bzrtpCallbacks_struct {
 #define BZRTP_ERROR_CHANNELALREADYSTARTED			0x0100
 #define BZRTP_ERROR_CACHEDISABLED				0x0200
 #define BZRTP_ERROR_CACHEMIGRATIONFAILED			0x0400
+#define BZRTP_ERROR_CACHE_PEERNOTFOUND				0x0800
 
 /* channel status definition */
 #define BZRTP_CHANNEL_NOTFOUND						0x1000
@@ -183,12 +195,29 @@ typedef struct bzrtpCallbacks_struct {
 #define BZRTP_CACHE_SETUP		0x2000
 #define BZRTP_CACHE_UPDATE		0x2001
 #define BZRTP_CACHE_DATA_NOTFOUND	0x2002
+#define BZRTP_CACHE_PEER_STATUS_UNKNOWN 0x2010
+#define BZRTP_CACHE_PEER_STATUS_VALID   0x2011
+#define BZRTP_CACHE_PEER_STATUS_INVALID 0x2012
+
+/* cache function error codes */
+#define BZRTP_ZIDCACHE_INVALID_CONTEXT		0x2101
+#define BZRTP_ZIDCACHE_INVALID_CACHE		0x2102
+#define BZRTP_ZIDCACHE_UNABLETOUPDATE		0x2103
+#define BZRTP_ZIDCACHE_UNABLETOREAD		0x2104
+#define BZRTP_ZIDCACHE_BADINPUTDATA		0x2105
+#define BZRTP_ZIDCACHE_RUNTIME_CACHELESS	0x2110
+
 /**
  * @brief bzrtpContext_t The ZRTP engine context
  * Store current state, timers, HMAC and encryption keys
 */
 typedef struct bzrtpContext_struct bzrtpContext_t;
 
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * Create context structure and initialise it
  *
@@ -208,6 +237,8 @@ BZRTP_EXPORT bzrtpContext_t *bzrtp_createBzrtpContext(void);
  */
 BZRTP_EXPORT int bzrtp_initBzrtpContext(bzrtpContext_t *context, uint32_t selfSSRC);
 
+BZRTP_EXPORT void bzrtp_resetBzrtpContext(bzrtpContext_t *context);
+
 /**
  * Free memory of context structure to a channel, if all channels are freed, free the global zrtp context
  * @param[in]	context		Context hosting the channel to be destroyed.(note: the context zrtp context itself is destroyed with the last channel)
@@ -219,7 +250,7 @@ BZRTP_EXPORT int bzrtp_destroyBzrtpContext(bzrtpContext_t *context, uint32_t sel
 
 /**
  * @brief Allocate a function pointer to the callback function identified by his id 
- * @param[in/out]	context				The zrtp context to set the callback function
+ * @param[in,out]	context				The zrtp context to set the callback function
  * @param[in] 		cbs 				A structure containing all the callbacks to supply.
  *
  * @return 0 on success
@@ -227,12 +258,27 @@ BZRTP_EXPORT int bzrtp_destroyBzrtpContext(bzrtpContext_t *context, uint32_t sel
 */
 BZRTP_EXPORT int bzrtp_setCallbacks(bzrtpContext_t *context, const bzrtpCallbacks_t *cbs);
 
+/**
+ * @brief Set the pointer allowing cache access,
+ * this version of the function get a mutex to lock the cache when accessing it
+ *
+ * @param[in,out]	context			The ZRTP context we're dealing with
+ * @param[in]		zidCache		Used by internal function to access cache: turn into a sqlite3 pointer if cache is enabled
+ * @param[in]		selfURI			Local URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
+ * @param[in]		peerURI			Peer URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
+ * @param[in]		zidCacheMutex		Points to a mutex used to lock zidCache database access
+ *
+ * @return 0 or BZRTP_CACHE_SETUP(if cache is populated by this call) on success, error code otherwise
+*/
+BZRTP_EXPORT int bzrtp_setZIDCache_lock(bzrtpContext_t *context, void *zidCache, const char *selfURI, const char *peerURI, bctbx_mutex_t *zidCacheMutex);
+
 /**
  * @brief Set the pointer allowing cache access
  *
- * @param[in]	zidCachePointer		Used by internal function to access cache: turn into a sqlite3 pointer if cache is enabled
- * @param[in]   selfURI			Local URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
- * @param[in]   peerURI			Peer URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
+ * @param[in,out]	context			The ZRTP context we're dealing with
+ * @param[in]		zidCache		Used by internal function to access cache: turn into a sqlite3 pointer if cache is enabled
+ * @param[in]   	selfURI			Local URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
+ * @param[in]   	peerURI			Peer URI used for this communication, needed to perform cache operation, NULL terminated string, duplicated by this function
  *
  * @return 0 or BZRTP_CACHE_SETUP(if cache is populated by this call) on success, error code otherwise
 */
@@ -241,7 +287,7 @@ BZRTP_EXPORT int bzrtp_setZIDCache(bzrtpContext_t *context, void *zidCache, cons
 /**
  * @brief Set the client data pointer in a channel context
  * This pointer is returned to the client by the callbacks function, used to store associated contexts (RTP session)
- * @param[in/out]	zrtpContext		The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext		The ZRTP context we're dealing with
  * @param[in]		selfSSRC		The SSRC identifying the channel to be linked to the client Data
  * @param[in]		clientData		The clientData pointer, casted to a (void *)
  *
@@ -253,7 +299,7 @@ BZRTP_EXPORT int bzrtp_setClientData(bzrtpContext_t *zrtpContext, uint32_t selfS
 /**
  * @brief Add a channel to an existing context
  *
- * @param[in/out]	zrtpContext	The zrtp context who will get the additionnal channel
+ * @param[in,out]	zrtpContext	The zrtp context who will get the additionnal channel
  * @param[in]		selfSSRC	The SSRC given to the channel context
  *
  * @return 0 on succes, error code otherwise
@@ -265,7 +311,7 @@ BZRTP_EXPORT int bzrtp_addChannel(bzrtpContext_t *zrtpContext, uint32_t selfSSRC
  * @brief Start the state machine of the specified channel
  * To be able to start an addional channel, we must be in secure state
  *
- * @param[in/out]	zrtpContext			The ZRTP context hosting the channel to be started
+ * @param[in,out]	zrtpContext			The ZRTP context hosting the channel to be started
  * @param[in]		selfSSRC			The SSRC identifying the channel to be started(will start sending Hello packets and listening for some)
  *
  * @return			0 on succes, error code otherwise
@@ -275,7 +321,7 @@ BZRTP_EXPORT int bzrtp_startChannelEngine(bzrtpContext_t *zrtpContext, uint32_t
 /**
  * @brief Send the current time to a specified channel, it will check if it has to trig some timer
  *
- * @param[in/out]	zrtpContext			The ZRTP context hosting the channel
+ * @param[in,out]	zrtpContext			The ZRTP context hosting the channel
  * @param[in]		selfSSRC			The SSRC identifying the channel
  * @param[in]		timeReference		The current time in ms
  *
@@ -286,7 +332,7 @@ BZRTP_EXPORT int bzrtp_iterate(bzrtpContext_t *zrtpContext, uint32_t selfSSRC, u
 /**
  * @brief Process a received message
  *
- * @param[in/out]	zrtpContext				The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext				The ZRTP context we're dealing with
  * @param[in]		selfSSRC				The SSRC identifying the channel receiving the message
  * @param[in]		zrtpPacketString		The packet received
  * @param[in]		zrtpPacketStringLength	Length of the packet in bytes
@@ -298,14 +344,14 @@ BZRTP_EXPORT int bzrtp_processMessage(bzrtpContext_t *zrtpContext, uint32_t self
 /**
  * @brief Called by user when the SAS has been verified
  *
- * @param[in/out]	zrtpContext				The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext				The ZRTP context we're dealing with
  */
 BZRTP_EXPORT void bzrtp_SASVerified(bzrtpContext_t *zrtpContext); 
 
 /**
  * @brief Called by user when the SAS has been set to unverified
  *
- * @param[in/out]	zrtpContext				The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext				The ZRTP context we're dealing with
  */
 BZRTP_EXPORT void bzrtp_resetSASVerified(bzrtpContext_t *zrtpContext);
 
@@ -315,7 +361,7 @@ BZRTP_EXPORT void bzrtp_resetSASVerified(bzrtpContext_t *zrtpContext);
  *  - when in responder role, zrtp engine only answer to packets sent by the initiator.
  *  - if we are still in discovery phase, Hello or Commit packets will be resent.
  *
- * @param[in/out]	zrtpContext				The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext				The ZRTP context we're dealing with
  * @param[in]		selfSSRC				The SSRC identifying the channel to reset
  *
  * return 0 on success, error code otherwise
@@ -325,7 +371,7 @@ BZRTP_EXPORT int bzrtp_resetRetransmissionTimer(bzrtpContext_t *zrtpContext, uin
 /**
  * @brief Get the supported crypto types
  *
- * @param[int]		zrtpContext				The ZRTP context we're dealing with
+ * @param[in]		zrtpContext				The ZRTP context we're dealing with
  * @param[in]		algoType				mapped to defines, must be in [ZRTP_HASH_TYPE, ZRTP_CIPHERBLOCK_TYPE, ZRTP_AUTHTAG_TYPE, ZRTP_KEYAGREEMENT_TYPE or ZRTP_SAS_TYPE]
  * @param[out]		supportedTypes			mapped to uint8_t value of the 4 char strings giving the supported types as string according to rfc section 5.1.2 to 5.1.6
  *
@@ -336,7 +382,7 @@ BZRTP_EXPORT uint8_t bzrtp_getSupportedCryptoTypes(bzrtpContext_t *zrtpContext,
 /**
  * @brief set the supported crypto types
  *
- * @param[int/out]	zrtpContext				The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext				The ZRTP context we're dealing with
  * @param[in]		algoType				mapped to defines, must be in [ZRTP_HASH_TYPE, ZRTP_CIPHERBLOCK_TYPE, ZRTP_AUTHTAG_TYPE, ZRTP_KEYAGREEMENT_TYPE or ZRTP_SAS_TYPE]
  * @param[in]		supportedTypes			mapped to uint8_t value of the 4 char strings giving the supported types as string according to rfc section 5.1.2 to 5.1.6
  * @param[in]		supportedTypesCount		number of supported crypto types
@@ -346,7 +392,7 @@ BZRTP_EXPORT void bzrtp_setSupportedCryptoTypes(bzrtpContext_t *zrtpContext, uin
 /**
  * @brief Set the peer hello hash given by signaling to a ZRTP channel
  *
- * @param[in/out]	zrtpContext						The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext						The ZRTP context we're dealing with
  * @param[in]		selfSSRC						The SSRC identifying the channel
  * @param[in]		peerHelloHashHexString			A NULL terminated string containing the hexadecimal form of the hash received in signaling,
  * 													may contain ZRTP version as header.
@@ -360,7 +406,7 @@ BZRTP_EXPORT int bzrtp_setPeerHelloHash(bzrtpContext_t *zrtpContext, uint32_t se
 /**
  * @brief Get the self hello hash from ZRTP channel
  *
- * @param[in/out]	zrtpContext			The ZRTP context we're dealing with
+ * @param[in,out]	zrtpContext			The ZRTP context we're dealing with
  * @param[in]		selfSSRC			The SSRC identifying the channel
  * @param[out]		output				A NULL terminated string containing the hexadecimal form of the hash received in signaling,
  * 										contain ZRTP version as header. Buffer must be allocated by caller.
@@ -384,23 +430,61 @@ BZRTP_EXPORT int bzrtp_getSelfHelloHash(bzrtpContext_t *zrtpContext, uint32_t se
  */
 BZRTP_EXPORT int bzrtp_getChannelStatus(bzrtpContext_t *zrtpContext, uint32_t selfSSRC);
 
+
+/**
+ * @brief Set Auxiliary Secret for this channel(shall be used only on primary audio channel)
+ *   The given auxSecret is appended to any aux secret found in ZIDcache
+ *   This function must be called before reception of peerHello packet
+ *
+ * @param[in]		zrtpContext	The ZRTP context we're dealing with
+ * @param[in]		auxSecret	A buffer holding the auxiliary shared secret to use (see RFC 6189 section 4.3)
+ * @param[in]		auxSecretLength	lenght of the previous buffer
+ *
+ * @return 0 on success, error code otherwise
+ *
+ * @note The auxiliary shared secret mechanic is used by LIMEv2 for encryption security purposes but might be used for its original purpose in a regular
+ * ZRTP session if it becomes necessary in the future, or by another encryption engine for example. In that case the API will need an adaptation work.
+ */
+BZRTP_EXPORT int bzrtp_setAuxiliarySharedSecret(bzrtpContext_t *zrtpContext, const uint8_t *auxSecret, size_t auxSecretLength);
+
+/**
+ * @brief Get the ZRTP auxiliary shared secret mismatch status
+ * @param[in]		zrtpContext			The ZRTP context we're dealing with
+ * @return	BZRTP_AUXSECRET_MATCH on match, BZRTP_AUXSECRET_MISMATCH on mismatch, BZRTP_AUXSECRET_UNSET if auxiliary shared secret is unused
+ */
+BZRTP_EXPORT uint8_t bzrtp_getAuxiliarySharedSecretMismatch(bzrtpContext_t *zrtpContext);
+
 /*** Cache related functions ***/
 /**
  * @brief Check the given sqlite3 DB and create requested tables if needed
  * 	Also manage DB schema upgrade
- * @param[in/out]	db	Pointer to the sqlite3 db open connection
+ * @param[in,out]	db	Pointer to the sqlite3 db open connection
  * 				Use a void * to keep this API when building cacheless
  *
  * @return 0 on success, BZRTP_CACHE_SETUP if cache was empty, BZRTP_CACHE_UPDATE if db structure was updated, error code otherwise
  */
-BZRTP_EXPORT int bzrtp_initCache(void *db);
+BZRTP_EXPORT BZRTP_DEPRECATED int bzrtp_initCache(void *db);
+
+/**
+ * @brief Check the given sqlite3 DB and create requested tables if needed
+ * 	Also manage DB schema upgrade
+ *
+ * this version of the function gets a mutex to lock the cache when accessing it
+ *
+ * @param[in,out]	db	Pointer to the sqlite3 db open connection
+ * 				Use a void * to keep this API when building cacheless
+ * @param[in]   zidCacheMutex	Points to a mutex used to lock zidCache database access, ignored if NULL
+ *
+ * @return 0 on success, BZRTP_CACHE_SETUP if cache was empty, BZRTP_CACHE_UPDATE if db structure was updated, error code otherwise
+ */
+BZRTP_EXPORT int bzrtp_initCache_lock(void *db, bctbx_mutex_t *zidCacheMutex);
 
 /**
  * @brief : retrieve ZID from cache
  * ZID is randomly generated if cache is empty or inexistant
  * ZID is randomly generated in case of cacheless implementation(db argument is NULL)
  *
- * @param[in/out] 	db		sqlite3 database(or NULL if we don't use cache at runtime)
+ * @param[in,out] 	db		sqlite3 database(or NULL if we don't use cache at runtime)
  * 					Use a void * to keep this API when building cacheless
  * @param[in]		selfURI		the sip uri of local user, NULL terminated string
  * @param[out]		selfZID		the ZID, retrieved from cache or randomly generated
@@ -408,22 +492,24 @@ BZRTP_EXPORT int bzrtp_initCache(void *db);
  *
  * @return		0 on success, or BZRTP_CACHE_DATA_NOTFOUND if no ZID matching the URI was found and no RNGContext is given to generate one
  */
-BZRTP_EXPORT int bzrtp_getSelfZID(void *db, const char *selfURI, uint8_t selfZID[12], bctbx_rng_context_t *RNGContext);
+BZRTP_EXPORT BZRTP_DEPRECATED int bzrtp_getSelfZID(void *db, const char *selfURI, uint8_t selfZID[12], bctbx_rng_context_t *RNGContext);
 
 /**
- * @brief get the cache internal id used to bind local uri(hence local ZID associated to it)<->peer uri/peer ZID.
- *	Providing a valid local URI(already present in cache), a peer ZID and peer URI will return the zuid creating it if needed
- *	Any pair ZID/sipURI shall identify an account on a device.
+ * @brief : retrieve ZID from cache
+ * ZID is randomly generated if cache is empty or inexistant
+ * ZID is randomly generated in case of cacheless implementation(db argument is NULL)
+ * this version of the function gets a mutex to lock the cache when accessing it
  *
- * @param[in/out]	db		the opened sqlite database pointer
- * @param[in]		selfURI		local URI, must be already associated to a ZID in the DB(association is performed by any call of getSelfZID on this URI)
- * @param[in]		peerURI		peer URI
- * @param[in]		peerZID		peer ZID
- * @param[out]		zuid		the internal db reference to the data row matching this particular pair of correspondant
+ * @param[in,out] 	db		sqlite3 database(or NULL if we don't use cache at runtime)
+ * 					Use a void * to keep this API when building cacheless
+ * @param[in]		selfURI		the sip uri of local user, NULL terminated string
+ * @param[out]		selfZID		the ZID, retrieved from cache or randomly generated
+ * @param[in]		RNGContext	A RNG context used to generate ZID if needed
+ * @param[in]  		zidCacheMutex	Points to a mutex used to lock zidCache database access, ignored if NULL
  *
- * @return 0 on success, error code otherwise
+ * @return		0 on success, or BZRTP_CACHE_DATA_NOTFOUND if no ZID matching the URI was found and no RNGContext is given to generate one
  */
-BZRTP_EXPORT int bzrtp_cache_getZuid(void *dbPointer, const char *selfURI, const char *peerURI, const uint8_t peerZID[12], int *zuid);
+BZRTP_EXPORT int bzrtp_getSelfZID_lock(void *db, const char *selfURI, uint8_t selfZID[12], bctbx_rng_context_t *RNGContext, bctbx_mutex_t *zidCacheMutex);
 
 /**
  * @brief Write(insert or update) data in cache, adressing it by zuid (ZID/URI binding id used in cache)
@@ -431,7 +517,7 @@ BZRTP_EXPORT int bzrtp_cache_getZuid(void *dbPointer, const char *selfURI, const
  *		All three arrays must be the same lenght: columnsCount
  * 		If the row isn't present in the given table, it will be inserted
  *
- * @param[in/out]	dbPointer	Pointer to an already opened sqlite db
+ * @param[in,out]	dbPointer	Pointer to an already opened sqlite db
  * @param[in]		zuid		The DB internal id to adress the correct row(binding between local uri and peer ZID+URI)
  * @param[in]		tableName	The name of the table to write in the db, must already exists. Null terminated string
  * @param[in]		columns		An array of null terminated strings containing the name of the columns to update
@@ -441,7 +527,27 @@ BZRTP_EXPORT int bzrtp_cache_getZuid(void *dbPointer, const char *selfURI, const
  *
  * @return 0 on succes, error code otherwise
  */
-BZRTP_EXPORT int bzrtp_cache_write(void *dbPointer, int zuid, char *tableName, char **columns, uint8_t **values, size_t *lengths, uint8_t columnsCount);
+BZRTP_EXPORT BZRTP_DEPRECATED int bzrtp_cache_write(void *dbPointer, int zuid, const char *tableName, const char **columns, uint8_t **values, size_t *lengths, uint8_t columnsCount);
+
+/**
+ * @brief Write(insert or update) data in cache, adressing it by zuid (ZID/URI binding id used in cache)
+ * 		Get arrays of column names, values to be inserted, lengths of theses values
+ *		All three arrays must be the same lenght: columnsCount
+ * 		If the row isn't present in the given table, it will be inserted
+ * this version of the function gets a mutex to lock the cache when accessing it
+ *
+ * @param[in,out]	dbPointer	Pointer to an already opened sqlite db
+ * @param[in]		zuid		The DB internal id to adress the correct row(binding between local uri and peer ZID+URI)
+ * @param[in]		tableName	The name of the table to write in the db, must already exists. Null terminated string
+ * @param[in]		columns		An array of null terminated strings containing the name of the columns to update
+ * @param[in]		values		An array of buffers containing the values to insert/update matching the order of columns array
+ * @param[in]		lengths		An array of integer containing the lengths of values array buffer matching the order of columns array
+ * @param[in]		columnsCount	length common to columns,values and lengths arrays
+ * @param[in]		zidCacheMutex	Points to a mutex used to lock zidCache database access, ignored if NULL
+ *
+ * @return 0 on succes, error code otherwise
+ */
+BZRTP_EXPORT int bzrtp_cache_write_lock(void *dbPointer, int zuid, const char *tableName, const char **columns, uint8_t **values, size_t *lengths, uint8_t columnsCount, bctbx_mutex_t *zidCacheMutex);
 
 /**
  * @brief Read data from specified table/columns from cache adressing it by zuid (ZID/URI binding id used in cache)
@@ -449,7 +555,7 @@ BZRTP_EXPORT int bzrtp_cache_write(void *dbPointer, int zuid, char *tableName, c
  *		Produce an array of values(uint8_t arrays) and a array of corresponding lengths
  *		Values memory is allocated by this function and must be freed by caller
  *
- * @param[in/out]	dbPointer	Pointer to an already opened sqlite db
+ * @param[in,out]	dbPointer	Pointer to an already opened sqlite db
  * @param[in]		zuid		The DB internal id to adress the correct row(binding between local uri and peer ZID+URI)
  * @param[in]		tableName	The name of the table to read in the db. Null terminated string
  * @param[in]		columns		An array of null terminated strings containing the name of the columns to read, the array's length  is columnsCount
@@ -459,14 +565,34 @@ BZRTP_EXPORT int bzrtp_cache_write(void *dbPointer, int zuid, char *tableName, c
  *
  * @return 0 on succes, error code otherwise
  */
-BZRTP_EXPORT int bzrtp_cache_read(void *dbPointer, int zuid, char *tableName, char **columns, uint8_t **values, size_t *lengths, uint8_t columnsCount);
+BZRTP_EXPORT BZRTP_DEPRECATED int bzrtp_cache_read(void *dbPointer, int zuid, const char *tableName, const char **columns, uint8_t **values, size_t *lengths, uint8_t columnsCount);
+
+/**
+ * @brief Read data from specified table/columns from cache adressing it by zuid (ZID/URI binding id used in cache)
+ * 		Get arrays of column names, values to be read, and the number of colums to be read
+ *		Produce an array of values(uint8_t arrays) and a array of corresponding lengths
+ *		Values memory is allocated by this function and must be freed by caller
+ * this version of the function gets a mutex to lock the cache when accessing it
+ *
+ * @param[in,out]	dbPointer	Pointer to an already opened sqlite db
+ * @param[in]		zuid		The DB internal id to adress the correct row(binding between local uri and peer ZID+URI)
+ * @param[in]		tableName	The name of the table to read in the db. Null terminated string
+ * @param[in]		columns		An array of null terminated strings containing the name of the columns to read, the array's length  is columnsCount
+ * @param[out]		values		An array of uint8_t pointers, each one will be allocated to the read value and they must be freed by caller
+ * @param[out]		lengths		An array of integer containing the lengths of values array buffer read
+ * @param[in]		columnsCount	length common to columns,values and lengths arrays
+ * @param[in]		zidCacheMutex	Points to a mutex used to lock zidCache database access, ignored if NULL
+ *
+ * @return 0 on succes, error code otherwise
+ */
+BZRTP_EXPORT int bzrtp_cache_read_lock(void *dbPointer, int zuid, const char *tableName, const char **columns, uint8_t **values, size_t *lengths, uint8_t columnsCount, bctbx_mutex_t *zidCacheMutex);
 
 /**
  * @brief Perform migration from xml version to sqlite3 version of cache
  *	Warning: new version of cache associate a ZID to each local URI, the old one did not
  *		the migration function will associate any data in the cache to the sip URI given in parameter which shall be the default URI
- * @param[in]		cacheXml	a pointer to an xmlDocPtr structure containing the old cache to be migrated
- * @param[in/out]	cacheSqlite	a pointer to an sqlite3 structure containing a cache initialised using bzrtp_cache_init function
+ * @param[in]		cacheXmlPtr	a pointer to an xmlDocPtr structure containing the old cache to be migrated
+ * @param[in,out]	cacheSqlite	a pointer to an sqlite3 structure containing a cache initialised using bzrtp_cache_init function
  * @param[in]		selfURI		default sip URI for this end point, NULL terminated char
  *
  * @return	0 on success, BZRTP_ERROR_CACHEDISABLED when bzrtp was not compiled with cache enabled, BZRTP_ERROR_CACHEMIGRATIONFAILED on error during migration
@@ -482,10 +608,42 @@ BZRTP_EXPORT int bzrtp_cache_migration(void *cacheXmlPtr, void *cacheSqlite, con
  * @param[in]		label			Label used in the KDF
  * @param[in]		labelLength		Length of previous buffer
  * @param[out]		derivedKey		Buffer to store the derived key
- * @param[in/out]	derivedKeyLength	Length of previous buffer(updated to fit actual length of data produced if too long)
+ * @param[in,out]	derivedKeyLength	Length of previous buffer(updated to fit actual length of data produced if too long)
  *
  * @return 0 on succes, error code otherwise
  */
 BZRTP_EXPORT int bzrtp_exportKey(bzrtpContext_t *zrtpContext, char *label, size_t labelLength, uint8_t *derivedKey, size_t *derivedKeyLength);
 
+/**
+ * @brief Retrieve from bzrtp cache the trust status(based on the previously verified flag) of a peer URI
+ *
+ * This function will return the SAS validation status of the active device
+ * associated to the given peerURI.
+ *
+ * Important note about the active device:
+ * - any ZRTP exchange with a peer device will set it to be the active one for its sip:uri
+ * - the concept of active device is shared between local accounts if there are several of them, it means that :
+ *       - if you have several local users on your device, each of them may have an entry in the ZRTP cache with a particular peer sip:uri (if they ever got in contact with it) but only one of this entry is set to active
+ *       - this function will return the status associated to the last updated entry without any consideration for the local users it is associated with
+ * - any call where the SAS was neither accepted or rejected will not update the trust status but will set as active device for the peer sip:uri the one involved in the call
+ *
+ * This function is intended for use in a mono-device environment.
+ *
+ * @param[in]	dbPointer	Pointer to an already opened sqlite db
+ * @param[in]	peerURI		The peer sip:uri we're interested in
+ * @param[in]	zidCacheMutex	Points to a mutex used to lock zidCache database access, ignored if NULL
+ *
+ * @return one of:
+ *  - BZRTP_CACHE_PEER_STATUS_UNKNOWN : this uri is not present in cache OR during calls with the active device, SAS never was validated or rejected
+ *  	Note: once the SAS has been validated or rejected, the status will never return to UNKNOWN(unless you delete your cache)
+ *  - BZRTP_CACHE_PEER_STATUS_VALID : the active device status is set to valid
+ *  - BZRTP_CACHE_PEER_STATUS_INVALID : the active peer device status is set to invalid
+ *
+ */
+BZRTP_EXPORT int bzrtp_cache_getPeerStatus_lock(void *dbPointer, const char *peerURI, bctbx_mutex_t *zidCacheMutex);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* ifndef BZRTP_H */
